import * as D from 'win32-def/def';
import type * as T from 'win32-def/types';
export declare class DefWinspool implements T.LibDefBase {
    [x: string]: T.FnDefFullParams;
    static ClosePrinter: (D.Def.int32 | (D.Def.uint32 | D.Def.uint64)[])[];
    static EndDocPrinter: (D.Def.int32 | (D.Def.uint32 | D.Def.uint64)[])[];
    static EndPagePrinter: (D.Def.int32 | (D.Def.uint32 | D.Def.uint64)[])[];
    /**
     * Enumerates available printers, print servers, domains, or print providers.
     * using multipleChoice to accept payload `pPrinterEnum` depends on `Level`
     * @link https://learn.microsoft.com/en-us/windows/win32/printdocs/enumprinters
     */
    static EnumPrintersW: (D.Def.int32 | string[])[];
    /**
     * Enumerates the print processors installed on the specified server.
     * @docs https://learn.microsoft.com/en-us/windows/win32/printdocs/enumprintprocessors
     */
    static EnumPrintProcessorsW: (D.Def.int32 | string[])[];
    /**
     * Enumerates the data types that a specified print processor supports.
     * @link https://learn.microsoft.com/en-us/windows/win32/printdocs/enumprintprocessordatatypes
     */
    static EnumPrintProcessorDatatypesW: (D.Def.int32 | string[])[];
    /** https://learn.microsoft.com/en-us/windows/win32/printdocs/getdefaultprinter */
    static GetDefaultPrinterW: (D.Def.int32 | string[])[];
    /**
     * Retrieves information about a specified print job
     * https://learn.microsoft.com/en-us/windows/win32/printdocs/getjob
     * @param pJob JOB_INFO_1 or a JOB_INFO_2
     */
    static GetJobW: (D.Def.int32 | string[])[];
    /**
     * Retrieves information about a specified printer.
     * @link https://learn.microsoft.com/en-us/windows/win32/printdocs/getprinter
     * @link https://learn.microsoft.com/zh-cn/windows/win32/printdocs/getprinter
     */
    static GetPrinterW: readonly [D.Def.int32, readonly [D.Def.uint32 | D.Def.uint64, D.Def.uint32, readonly ["_Out_ PRINTER_INFO_1*", "_Out_ PRINTER_INFO_4*", "_Out_ PRINTER_INFO_5*", "_Out_ PRINTER_INFO_6*", "_Out_ PRINTER_INFO_8*", "_Out_ PRINTER_INFO_9*"], D.Def.uint32, "_Out_ uint16_t*"]];
    /**
     * Retrieves a handle to the specified printer or print server or other types of handles in the print subsystem.
     * @docs https://learn.microsoft.com/en-us/windows/win32/printdocs/openprinter
     * @docs https://learn.microsoft.com/zh-cn/windows/win32/printdocs/openprinter
     */
    static OpenPrinterW: (D.Def.int32 | string[])[];
    /**
     * Notifies the print spooler that a document is to be spooled for printing.
     * @param pDocInfo A pointer to a DOC_INFO_1 structure that describes the document to print.
     * @link https://learn.microsoft.com/en-us/windows/win32/printdocs/startdocprinter
     */
    static StartDocPrinterW: (D.Def.uint32 | (D.Def.uint32 | D.Def.uint64 | "DOC_INFO_1*")[])[];
    /**
     * Notifies the spooler that a page is about to be printed on the specified printer.
     * @docs https://learn.microsoft.com/zh-cn/windows/win32/printdocs/startpageprinter
     */
    static StartPagePrinter: (D.Def.int32 | (D.Def.uint32 | D.Def.uint64)[])[];
    /**
     * Notifies the print spooler that data should be written to the specified printer.
     * @note Only supports GDI printing and must not be used for XPS printing
     * @link https://learn.microsoft.com/zh-cn/windows/win32/printdocs/writeprinter
     */
    static WritePrinter: (D.Def.int32 | string[])[];
}
//# sourceMappingURL=api.def.d.ts.map